﻿namespace Slash.Unity.DataBind.Core.Data
{
    using System;
    using System.ComponentModel;

    /// <summary>
    ///     Value observer that works with <see cref="INotifyPropertyChanged" /> as a parent object.
    /// </summary>
    public class NotifyPropertyChangedNodeValueObserver : INodeValueObserver
    {
        private readonly string propertyName;

        /// <summary>
        ///   Constructor.
        /// </summary>
        /// <param name="parentObject">Parent object.</param>
        /// <param name="propertyName">Property name to observe.</param>
        public NotifyPropertyChangedNodeValueObserver(INotifyPropertyChanged parentObject, string propertyName)
        {
            this.propertyName = propertyName;
            parentObject.PropertyChanged += this.OnPropertyChanged;
        }

        /// <inheritdoc />
        public event Action ValueChanged;

        private void OnPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == this.propertyName)
            {
                this.OnValueChanged();
            }
        }

        private void OnValueChanged()
        {
            var handler = this.ValueChanged;
            if (handler != null)
            {
                handler();
            }
        }
    }
}